/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "mozilla/ArrayUtils.h"

#include "nsCOMPtr.h"
#include "nsKDEShellService.h"
#include "nsShellService.h"
#include "nsKDEUtils.h"
#include "nsIPrefService.h"
#include "nsIProcess.h"
#include "nsIFile.h"
#include "nsServiceManagerUtils.h"
#include "nsComponentManagerUtils.h"
#include "nsIMutableArray.h"
#include "nsISupportsPrimitives.h"
#include "nsArrayUtils.h"

using namespace mozilla;

nsresult
nsKDEShellService::Init()
{
    if( !nsKDEUtils::kdeSupport())
        return NS_ERROR_NOT_AVAILABLE;
    return NS_OK;
}

NS_IMPL_ISUPPORTS(nsKDEShellService, nsIGNOMEShellService, nsIShellService)

NS_IMETHODIMP
nsKDEShellService::IsDefaultBrowser(bool aForAllTypes,
                                    bool* aIsDefaultBrowser)
{
    *aIsDefaultBrowser = false;

    nsCOMPtr<nsIMutableArray> command = do_CreateInstance( NS_ARRAY_CONTRACTID );
    if (!command)
        return NS_ERROR_FAILURE;

    nsCOMPtr<nsISupportsCString> str = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    if (!str)
        return NS_ERROR_FAILURE;

    str->SetData( NS_LITERAL_CSTRING( "ISDEFAULTBROWSER" ));
    command->AppendElement( str );

    if( nsKDEUtils::command( command ))
        *aIsDefaultBrowser = true;
    return NS_OK;
}

NS_IMETHODIMP
nsKDEShellService::SetDefaultBrowser(bool aClaimAllTypes,
                                     bool aForAllUsers)
{
    nsCOMPtr<nsIMutableArray> command = do_CreateInstance( NS_ARRAY_CONTRACTID );
    if (!command)
        return NS_ERROR_FAILURE;

    nsCOMPtr<nsISupportsCString> cmdstr = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    nsCOMPtr<nsISupportsCString> paramstr = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    if (!cmdstr || !paramstr)
        return NS_ERROR_FAILURE;

    cmdstr->SetData( NS_LITERAL_CSTRING( "SETDEFAULTBROWSER" ));
    command->AppendElement( cmdstr );

    paramstr->SetData( aClaimAllTypes ? NS_LITERAL_CSTRING( "ALLTYPES" ) : NS_LITERAL_CSTRING( "NORMAL" ));
    command->AppendElement( paramstr );

    return nsKDEUtils::command( command ) ? NS_OK : NS_ERROR_FAILURE;
}

NS_IMETHODIMP
nsKDEShellService::GetCanSetDesktopBackground(bool* aResult)
{
  *aResult = true;
  return NS_OK;
}

NS_IMETHODIMP
nsKDEShellService::SetDesktopBackground(dom::Element* aElement,
                                        int32_t aPosition,
                                        const nsACString& aImageName)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
nsKDEShellService::GetDesktopBackgroundColor(PRUint32 *aColor)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
nsKDEShellService::SetDesktopBackgroundColor(PRUint32 aColor)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

NS_IMETHODIMP
nsKDEShellService::OpenApplication(PRInt32 aApplication)
{
    nsCOMPtr<nsIMutableArray> command = do_CreateInstance( NS_ARRAY_CONTRACTID );
    if (!command)
        return NS_ERROR_FAILURE;

    nsCOMPtr<nsISupportsCString> str = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    if (!str)
        return NS_ERROR_FAILURE;

    if( aApplication == APPLICATION_MAIL )
        str->SetData( NS_LITERAL_CSTRING( "OPENMAIL" ));
    else if( aApplication == APPLICATION_NEWS )
        str->SetData( NS_LITERAL_CSTRING( "OPENNEWS" ));
    else
        return NS_ERROR_NOT_IMPLEMENTED;

    command->AppendElement( str );
    return nsKDEUtils::command( command ) ? NS_OK : NS_ERROR_FAILURE;
}

NS_IMETHODIMP
nsKDEShellService::OpenApplicationWithURI(nsIFile* aApplication, const nsACString& aURI)
{
    nsCOMPtr<nsIMutableArray> command = do_CreateInstance( NS_ARRAY_CONTRACTID );
    if (!command)
        return NS_ERROR_FAILURE;

    nsCOMPtr<nsISupportsCString> cmdstr = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    nsCOMPtr<nsISupportsCString> appstr = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    nsCOMPtr<nsISupportsCString> uristr = do_CreateInstance( NS_SUPPORTS_CSTRING_CONTRACTID );
    if (!cmdstr || !appstr || !uristr)
        return NS_ERROR_FAILURE;

    cmdstr->SetData( NS_LITERAL_CSTRING( "RUN" ));
    command->AppendElement( cmdstr );
    nsAutoCString app;
    nsresult rv = aApplication->GetNativePath( app );
    NS_ENSURE_SUCCESS( rv, rv );
    appstr->SetData( app );
    command->AppendElement( appstr );
    uristr->SetData( aURI );
    command->AppendElement( uristr );
    return nsKDEUtils::command( command ) ? NS_OK : NS_ERROR_FAILURE;
}

